/***************************************************************************
  This is a library for the AMG88xx GridEYE 8x8 IR camera

  This sketch tries to read the pixels from the sensor

  Designed specifically to work with the Adafruit AMG88 breakout
  ----> http://www.adafruit.com/products/3538

  These sensors use I2C to communicate. The device's I2C address is 0x69

  Adafruit invests time and resources providing this open source code,
  please support Adafruit andopen-source hardware by purchasing products
  from Adafruit!

  Written by Dean Miller for Adafruit Industries.
  BSD license, all text above must be included in any redistribution
 ***************************************************************************/

#include <Wire.h>
#include <Adafruit_AMG88xx.h>

int digitalWritePin1 = 2;   
int digitalWritePin2 = 13;
int val = 0;  // variable to store the value read

Adafruit_AMG88xx amg;

void setup() {
    Serial.begin(1000000);
    Serial.println(F("AMG88xx pixels"));

    bool status;

    
        // default settings
    status = amg.begin();
    amg.setMovingAverageMode(true);
    if (!status) {
        Serial.println("Could not find a valid AMG88xx sensor, check wiring!");
        while (1);
    }

    Serial.println("-- Pixels Test --");
    
    Serial.println();

    delay(100); // let sensor boot up

    pinMode(digitalWritePin1, OUTPUT);
    pinMode(digitalWritePin2, OUTPUT);
}

void loop() {
  float pixels[AMG88xx_PIXEL_ARRAY_SIZE];

  //read all the pixels
  amg.readPixels(pixels);

  //run thermalNav
  char c = thermalNav(pixels);

  if (c == 'F'){
    digitalWrite(digitalWritePin1, HIGH);
    digitalWrite(digitalWritePin2, HIGH);
  }

  else if (c == 'L'){
    digitalWrite(digitalWritePin1, HIGH);
    digitalWrite(digitalWritePin2, LOW);
  }

  else if (c == 'R'){
    digitalWrite(digitalWritePin1, LOW);
    digitalWrite(digitalWritePin2, HIGH);
  }

  else if (c == 'S'){
    digitalWrite(digitalWritePin1, LOW);
    digitalWrite(digitalWritePin2, LOW);
  }
}

char thermalNav(float pixels[AMG88xx_PIXEL_ARRAY_SIZE]){
  
  int const segments = 2;
  float const threshold = 25;
  float sumL = 0.00;
  float sumR = 0.00;
  float const turnThreshold = 4.0;
  

  // find the minimum and maximum values in the input array
  float min_val = pixels[0];
  float max_val = pixels[0];
  for (int i = 0; i < AMG88xx_PIXEL_ARRAY_SIZE; i++) {
    if (pixels[i] < min_val) {
      min_val = pixels[i];
    }
    if (pixels[i] > max_val) {
      max_val = pixels[i];
    }
    Serial.print(String(pixels[i])+",");
  }
  Serial.print('\n');

  // arrange the input array into an 8x8 matrix
  float matrix[8][8] = {};
  for (int i = 0; i < 8; i++) {
    for (int j = 0; j < 8; j++) {
      matrix[i][j] = pixels[(i)*8 + (j)];
    }
  }

  // sum values per half of field of matrix
  for (int i = 0; i < 8; i++) {
    for (int j = 0; j < 4; j++) {
      if (matrix[i][j] < threshold){
        matrix[i][j] = 0.00;
      }
      sumL += matrix[i][j];
    }
    for (int j = 4; j < 8; j++) {
      if (matrix[i][j] < threshold){
        matrix[i][j] = 0.00;
      }
      sumR += matrix[i][j];
    }
  }

  // check case and return result

  // right signal
  if (sumR - sumL > turnThreshold && (sumL+sumR) < turnThreshold*100){
    return 'R';
  }

  // left signal
  else if (sumL - sumR > turnThreshold && (sumL+sumR) < turnThreshold*100){
    return 'L';
  }

  // middle forward signal
  else if ((sumL+sumR) > turnThreshold){
    return 'F';
  }

  // low / search signal
  else {
    return 'S';
  }
}